[§3] X.509 library - (sagittarius crypto x509) {#sagittarius.crypto.x509}
------------------------------------------------------

The X.509 library provides X.509 certificate, CRL, CSR and some
other related operations.

###### [!Library] `(sagittarius crypto x509)`

The X.509 library, this library exports the procedures listed
below sections.

### [§4] X.509 certificate

###### [!Function] `x509-certificate?` _obj_

Returns `#t` if the given _obj_ is X.509 certificate, otherwise `#f`.

###### [!Function] `bytevector->x509-certificate` (_bv_ `bygevector?`)

Converts the given bytevector _bv_ to X.509 certificate. The bytevector
must be a valid X.509 certificate structure otherwise an error is signalled.

###### [!Function] `read-x509-certificate` (_bin_ (and `input-port?` `binary-port?`))

Reads X.509 certificate from the given binary input port _bin_. The data
must be a valid X.509 certificate structure otherwise an error is signalled.

###### [!Function] `x509-certificate->bytevector` (_certificate_ `x509-certificate?`)

Converts the given X.509 certificate _certificate_ to a bytevector.

###### [!Function] `write-x509-certificate` (_certificate_ `x509-certificate?`) :optional _out_

Writes the given X.509 certificate into _out_ if specified, otherwise 
`(current-output-port)`.

###### [!Function] `x509-certificate-expired?` (_certificate_ `x509-certificate?`) :optional when

Checks if the given X.509 certificate is expired on the given _when_
if specified, otherwise `(current-date)`.

###### [!Function] `x509-certificate-issuer-dn` (_certificate_ `x509-certificate?`)

Retrieves IssuerDn of the given _certificate_ as X.509 name.

###### [!Function] `x509-certificate-subject-dn` (_certificate_ `x509-certificate?`)

Retrieves SubjectDn of the given _certificate_ as X.509 name.

###### [!Function] `x509-certificate-public-key` (_certificate_ `x509-certificate?`)

Retrieves public key of the given _certificate_.

###### [!Function] `x509-certificate-validity` (_certificate_ `x509-certificate?`)

Retrieves validity of the given _certificate_ as X.509 validity.

###### [!Function] `x509-certificate-serial-number` (_certificate_ `x509-certificate?`)

Retrieves serial number of the given _certificate_.

###### [!Function] `x509-certificate-version` (_certificate_ `x509-certificate?`)

Retrieves version of the given _certificate_.  
The returning value indicates the version number, for example, 
v1 certificate then the procedure returns `1`.

###### [!Function] `x509-certificate-signature` (_certificate_ `x509-certificate?`)

Retrieves signature of the given _certificate_.

###### [!Function] `x509-certificate-signature-algorithm` (_certificate_ `x509-certificate?`)

Retrieves signature algorithm of the given _certificate_ as X.509
algorithm identifier.

###### [!Function] `x509-certificate-extensions` (_certificate_ `x509-certificate?`)

Retrieves extensions of the given _certificate_ as a list of X.509 extension.

###### [!Function] `validate-x509-certificate` (_certificate_ `x509-certificate?`) _validator_ _..._

Validates the given _certificate_ against the given _validator_.  
_validator_ must be a procedure accepts one argument, _certificate_.

###### [!Function] `x509-certificate-signature-validator` (_public-key_ `public-key?`)

Creates a signature validator.

###### [!Function] `x509-certificate-validity-validator` :optional _when_

Creates a validity validator.  
Optional argument _when_ specifies in which period of time the certificate
validity. Default value is `(current-date)`.

###### [!Function] `x509-certificate-signed-by?` (_certificate_ `x509-certificate?`) (_ca-certificate_ `x509-certificate?`)

Returns `t` if the given _certificate_ is signed by the _ca-certificate_,
otherwise `#t`.

###### [!Function] `x509-certificate-revoked?`  (_certificate_ `x509-certificate?`) (_ca-certificate_ `x509-certificate?`) (_crl_ `x509-certificate-revocation-list?`) :optional _date_

Checks if the given _certificate_ is revoked on the given _crl_.  
If the optional argument _date_ is provided, then the procedure checks on
the given date. Default value is `(current-date)`.

###### [!Function] `x509-certificate-template?` _obj_

Returns `#t` if the given _obj_ is X.509 certificate template, otherwise `#f`.

###### [!Macro] `x509-certificate-template-builder` (_field_ _value_) _..._

Builds an X.509 certificate template. _field_ and its _value_ must
be one or more of the followings:

`issuer-dn`
: X.509 name, represents IssuerDn. Required.

`subject-dn`
: X.509 name, represents SubjectDn. Required.

`serial-number`
: Integer, represents serial number. Required.

`not-before`
: Date, represents not before. Required.

`not-after`
: Date, represents not after. Required.

`public-key`
: Public key of the certificate. Required.

`issuer-unique-id`
: Bytevector, represents issuer unique Id. Optional.

`subject-unique-id`
: Bytevector, represents subject unique Id. Optional.

`extentions`
: List of X.509 extensions. Optional.

###### [!Function] `sign-x509-certificate-template` (_template_ `x509-certificate-template?`) (_aid_ (or `string?` `x509-algorithm-identifier?`)) (_private-key_ `private-key?`)

Signs the given _template_ with _aid_ algorithm and _private-key_, then
returns X.509 certificate.

### [§4] X.509 CSR

###### [!Function] `x509-certification-request?` _obj_

Returns `#t` if the given _obj_ is X.509 certificate, otherwise `#f`.

###### [!Function] `bytevector->x509-certification-request` (_bv_ `bygevector?`)

Converts the given bytevector _bv_ to X.509 CSR. The bytevector
must be a valid X.509 CSR otherwise an error is signalled.

###### [!Function] `read-x509-certification-request` (_bin_ (and `input-port?` `binary-port?`))

Reads X.509 CSR from the given binary input port _bin_. The data
must be a valid X.509 CSR structure otherwise an error is signalled.

###### [!Function] `x509-certification-request->bytevector` (_csr_ `x509-certification-request?`)

Converts the given X.509 CSR _csr_ to a bytevector.

###### [!Function] `write-x509-certification-request` (_csr_ `x509-certification-request?`) :optional _out_

Writes the given X.509 CSR into _out_ if specified, otherwise 
`(current-output-port)`.

###### [!Function] `x509-certification-request-version` (_csr_ `x509-certification-request?`)

Retrieves version of the given _csr_.  
The returning value indicates the version number, for example, 
v1 certificate then the procedure returns `1`.

###### [!Function] `x509-certification-request-subject` (_csr_ `x509-certification-request?`)

Retrieves SubjectDn of the given _csr_ as X.509 name.

###### [!Function] `x509-certification-request-public-key` (_csr_ `x509-certification-request?`)

Retrieves public key of the given _csr_.

###### [!Function] `x509-certification-request-attributes` (_csr_ `x509-certification-request?`)

Retrieves list of attributes of the given _csr_ as list of X.509 attributes.

###### [!Function] `x509-certification-request-signature` (_csr_ `x509-certification-request?`)

Retrieves signature of the given _csr_.

###### [!Function] `x509-certification-request-signature-algorithm` (_csr_ `x509-certification-request?`)

Retrieves signature algorithm of the given _csr_ as X.509
algorithm identifier.

###### [!Function] `validate-x509-certification-request` (_csr_ `x509-certification-request?`) _validator_ _..._

Validates the given _csr_ against the given _validator_.  
_validator_ must be a procedure accepts one argument, _csr_.

###### [!Function] `x509-certification-request-signature-validator`

X.509 CSR signature validator. It validates the signature of the target
X.509 CSR.
	    
###### [!Function] `x509-certification-request-template?` _obj_

Returns `#t` if the given _obj_ is X.509 CSR template, otherwise `#f`.

###### [!Macro] `x509-certification-request-template-builder` (_field_ _value) _..._

Builds an X.509 CSR template. _field_ and its _value_ must
be one or more of the followings:

`subject-dn`
: X.509 name, represents SubjectDn. Required.

`attributes`
: List of X.509 attributes. Optional.


###### [!Function] `sign-x509-certification-request-template` (_template_ `x509-certification-request-template?`) (_aid_ (or `string?` `x509-algorithm-identifier?`)) (_signing-key-pair_ `key-pair?`)

Signs the given _template_ with _aid_ algorithm and the private key of
the _signing-key-pair_, then returns X.509 CSR.  
The returning X.509 CSR contains the public key of the _signing-key-pair_.

### [§4] X.509 CRL

###### [!Function] `x509-certificate-revocation-list?` _obj_

Returns `#t` if the given _obj_ is X.509 CRL, otherwise `#f`.

###### [!Function] `bytevector->x509-certificate-revocation-list` (_bv_ `bygevector?`)

Converts the given bytevector _bv_ to X.509 CRL. The bytevector
must be a valid X.509 CRL structure otherwise an error is signalled.

###### [!Function] `read-x509-certificate-revocation-list` (_bin_ (and `input-port?` `binary-port?`))

Reads X.509 CRL from the given binary input port _bin_. The data
must be a valid X.509 CRL structure otherwise an error is signalled.

###### [!Function] `x509-certificate-revocation-list->bytevector` (_crl_ `x509-certificate-revocation-list?`)

Converts the given X.509 CRL _crl_ to a bytevector.

###### [!Function] `write-x509-certificate-revocation-list` (_crl_ `x509-certificate-revocation-list?`) :optional _out_

Writes the given X.509 CRL into _out_ if specified, otherwise 
`(current-output-port)`.

###### [!Function] `x509-certificate-revocation-list-issuer` (_crl_ `x509-certificate-revocation-list?`)

Retrieves IssuerDn of the given _crl_ as X.509 name.

###### [!Function] `x509-certificate-revocation-list-this-update` (_crl_ `x509-certificate-revocation-list?`)

Retrieves this update of the given _crl_ as date.

###### [!Function] `x509-certificate-revocation-list-next-update` (_crl_ `x509-certificate-revocation-list?`)

Retrieves next update of the given _crl_ as date.

###### [!Function] `x509-certificate-revocation-list-revoked-certificates` (_crl_ `x509-certificate-revocation-list?`)

Retrieves X.509 revoked certificate of the given _crl_ as list.

###### [!Function] `x509-certificate-revocation-list-crl-extensions` (_crl_ `x509-certificate-revocation-list?`)

Retrieves X.509 extensions of the given _crl_ as X.509 extension list.

###### [!Function] `x509-certificate-revocation-list-signature-algorithm` (_crl_ `x509-certificate-revocation-list?`)

Retrieves signature algorithm the given _crl_ as X.509 algorithm identifier.

###### [!Function] `x509-certificate-revocation-list-signature` (_crl_ `x509-certificate-revocation-list?`)

Retrieves signature of the given _crl_.

###### [!Function] `validate-x509-certificate-revocation-list` (_crl_ `x509-certificate-revocation-list?`) _validator_ _..._

Validates the given _crl_ with the given _validator_, if the validation failed,
then the procedure signals an error.

###### [!Function] `x509-certificate-revocation-list-signature-validator` (_public-key_ `public-key?`)

Creates a X.509 CRL validator which verifies the signature with the
given _public-key_.

###### [!Function] `x509-certificate-revocation-list-issuer-validator` (_issuer_ `x509-name?`)

Creates a X.509 CRL validator which checks if the issuer DN is
_issuer_ or not.

###### [!Function] `x509-revoked-certificate?` _obj_

Returns `#t` if the given _obj_ is X.509 revoked certificate, otherwise `#f`.

###### [!Function] `make-x509-revoked-certificate` (_serial-number_ `integer?`) (_revocation-date_ `date?`) :optional _extensions_

Creates a X.509 revoked certificate.

###### [!Function] `x509-revoked-certificate-serial-number` (_rc_ `x509-revoked-certificate?`)

Retrieves serial number of the given _rc_.

###### [!Function] `x509-revoked-certificate-revocation-date` (_rc_ `x509-revoked-certificate?`)

Retrieves revocation date of the given _rc_ as date.

###### [!Function] `x509-revoked-certificate-crl-entry-extensions` (_rc_ `x509-revoked-certificate?`)

Retrieves CRL entry extensions of the given _rc_ as X.509 extension list.

###### [!Function] `x509-certificate-revocation-list-template?` _obj_

Returns `#t` if the given _obj_ is X.509 CRL template, otherwise `#f`.

###### [!Function] `x509-certificate-revocation-list-template-builder` (_field_ _value_) _..._

Builds an X.509 CRL template. _field_ and its _value_ must be one or more
of the followings:

`issuer-dn`
: X.509 name, represents IssuerDn. Required.

`this-update`
: Date, represents this update. Required.

`next-update`
: Date, represents next update. Optional.

`revoked-certificate`
: A list of X.509 revoked certificate, represents revoked certificates.
  Required.

`crl-extensions`
: A list of X.509 extensions, represents CRL extensions. Optional.


###### [!Function] `sign-x509-certificate-revocation-list-template` (_template_ `x509-certificate-revocation-list-template?`) (_aid_ (or `string?` `x509-algorithm-identifier`)) (_private-key_ `private-key?`)

Signs the given _template_ with _aid_ algorithm and _private-key_, then
returns X.509 CRL.

### [§4] Distinguished name (DN)

###### [!Function] `x509-name?` _obj_

Returns `#t` if the given _obj_ is X.509 name, otherwise `#f`.

###### [!Function] `x509-name` _component_ _..._
###### [!Function] `list->x509-name` _list of component_

Creates X.509 name from the given *component*s.

A component must be a list of either:

- Symbol representation of well-known name, such as `CN`, and string value
- Or string representation of OID and string value.

Below are the supported symbols:

`CN`
: Common name

`L`
: Locality name

`ST`
: State or province name

`O`
: Origanisation name

`OU`
: Origanisation unit name

`STREET`
: Street address

`DC`
: Domain component

`UID`
: User ID

`Name`
: Name

`SURNAME`
: Surname

`GIVENNAME`
: Given name

`T`
: Title

`DN`
: DN qualifier

`SERIALNUMBER`
: Serial number

`Pseudonym`
: Pseudonym

`E`
: Email address


###### [!Function] `x509-name->string` (_name_ `x509-name?`)

Converts given _name_ to string representation.


### [§4] X.509 validity

###### [!Function] `x509-validity?` _obj_

Returns `#t` if the given _obj_ is X.509 validity, otherwise `#f`.

###### [!Function] `make-x509-validity` (_not-before_ `date?`) (_not-after_ `date?`)

Creates a X.509 validity of the range of _not-after_ and _not-after_.

###### [!Function] `x509-validity-not-before` (_validity_ `x509-validity?`)

Retrieves not before of _validity_ as date.

###### [!Function] `x509-validity-not-after` (_validity_ `x509-validity?`)

Retrieves not after of _validity_ as date.

### [§4] X.509 algorithm identifier

###### [!function] `x509-algorithm-identifier?` _obj_

returns `#t` if the given _obj_ is x.509 algorithm identifier,
otherwise `#f`.

###### [!function] `make-x509-algorithm-identifier` (_oid_ `string?`) :optional parameters

_oid_ must be a valid string representaion of oid.  
creates a x.509 algorithm identifier of _oid.  
if optional parameter is passed, then it has to be a valid x.509 algorithm
identifier parameter.

###### [!Function] `x509-algorithm-identifier-oid` (_aid_ `x509-algorithm-identifier?`)

Retrieves OID of _aid_.

###### [!Function] `x509-algorithm-identifier-parameters` (_aid_ `x509-algorithm-identifier?`)

Retrieves parameters of _aid_.

### [§4] X.509 subject public key info

###### [!Function] `subject-public-key-info?` _obj_

returns `#t` if the given _obj_ is subject public key info, otherwise `#f`.

###### [!Function] `subject-public-key-info->public-key` (_spki_ `subject-public-key-info?`)

Converts given _spki_ to a public key.

###### [!Function] `public-key->subject-public-key-info` (_public-key_ `public-key?`)

Converts given _public-key_ to a subject public key info.

###### [!Function] `bytevector->subject-public-key-info` (_bv_ `bytevector?`)

Converts given _bv_ to a subject public key info.  
The bytevector must be a valid SubjectPublicKeyInfo structure. Otherwise,
an error is signalled.

###### [!Function] `subject-public-key-info->bytevector` (_spki_ `subject-public-key-info?`)

Converts given _spki_ to a bytevector.


### [§4] X.509 attribute

###### [!Function] `x509-attribute?` _obj_

Returns `#t` if the given _obj_ is X.509 attribute, otherwise `#f`.

###### [!Function] `x509-attribute-type` (_attr_ `x509-attribute?`)

Retrieves type of the _attr_.

###### [!Function] `x509-attribute-values` (_attr_ `x509-attribute?`)

Retrieves values of the _attr_. The retrieved value may not be
usable as it is.

###### [!Function] `x509-attribute-of` _oid_

_oid_ must be a string representaion of OID or object identifier.  
Returns a predicate for a X.509 attribute list. The predicate
checks if the given X.509 attribute has _oid_ or not.

###### [!Function] `make-x509-challenge-password-attribute` (_password_ `string?`)

Creates PKCS#9 challenge password attribute of the value _password_.

###### [!Function] `make-x509-extension-request-attribute` _x509-extension_ _..._

_x509-extension_ must be X.509 extension.  
Creates PKCS#9 extension request attribute of the value *x509-extension*s.

###### [!Function] `x509-attribute->x509-extension-list` (_attr_ `x509-attribute`)

Converts PKCS#9 extension request attribute to list of X.509 extensions.  
If the _attr_ does not have appropriate attribute type, the procedure
signals an error.

###### [!Object identifier] `*pkcs-9:channelge-password*`
###### [!Object identifier] `*pkcs-9:extenion-request*`

The above object identifiers can be use for `x509-attribute-of` procedure.

### [§4] X.509 extension

###### [!Function] `x509-extension?` _obj_

Returns `#t` if the given _obj_ is X.509 extension, otherwise `#f`.

###### [!Function] `x509-extension-id` (_ext_ `x509-extension?`)

Retrieves id of the _ext_.

###### [!Function] `x509-extension-critical?` (_ext_ `x509-extension?`)

Retrieves critical of the _ext_.

###### [!Function] `x509-extension-value` (_ext_ `x509-extension?`)

Retrieves value of the _ext_.

###### [!Object identifier] `*extension:authority-key-identifier*`
###### [!Object identifier] `*extension:subject-key-identifier*`
###### [!Object identifier] `*extension:key-usage*`
###### [!Object identifier] `*extension:private-key-usage-period*`
###### [!Object identifier] `*extension:certificate-policies*`
###### [!Object identifier] `*extension:policy-mappings*`
###### [!Object identifier] `*extension:subject-alt-name*`
###### [!Object identifier] `*extension:issuer-alt-name*`
###### [!Object identifier] `*extension:subject-directory-attributes*`
###### [!Object identifier] `*extension:basic-constraints*`
###### [!Object identifier] `*extension:name-constraints*`
###### [!Object identifier] `*extension:policy-constraints*`
###### [!Object identifier] `*extension:ext-key-usage*`
###### [!Object identifier] `*extension:crl-distribution-points*`
###### [!Object identifier] `*extension:inhibit-any-policy*`
###### [!Object identifier] `*extension:freshest-crl*`
###### [!Object identifier] `*extension:authority-info-access*`
###### [!Object identifier] `*extension:subject-info-access*`
###### [!Object identifier] `*extension:crl-number*`
###### [!Object identifier] `*extension:delta-crl-indicator*`
###### [!Object identifier] `*extension:issuing-distribution-point*`
###### [!Object identifier] `*extension:crl-reason*`
###### [!Object identifier] `*extension:certificate-issuer*`
###### [!Object identifier] `*extension:hold-instruction-code*`
###### [!Object identifier] `*extension:invalidity-date*`

Object identifiers of X.509 extension. The name must be descritive enough
to see which extension's OID it is.

###### [!Function] `x509-extension-of` _oid_

_oid_ must be a string representaion of OID or object identifier.  
Creates a predicate for a list of X.509 extensions.

<!-- // TBD
x509-general-name?
rfc822-name->general-name
dns-name->general-name
ip-address->general-name
uri->general-name
directory-name->general-name
registered-id->general-name

x509-general-names?
list->x509-general-names
x509-general-names

make-x509-subject-alternative-name-extension
x509-subject-alternative-name-extension->x509-general-names
make-x509-issuer-alternative-name-extension
x509-issuer-alternative-name-extension->x509-general-names

x509-authority-key-identifier?
make-x509-authority-key-identifier
x509-authority-key-identifier-key-identifier
x509-authority-key-identifier-authority-cert-issuer
x509-authority-key-identifier-authority-cert-serial-number

make-x509-authority-key-identifier-extension
x509-authority-key-identifier-extension->x509-authority-key-identifier

make-x509-subject-key-identifier-extension
x509-subject-key-identifier-extension->subject-key-identifier

x509-key-usages 
x509-key-usages?
make-x509-key-usage-extension
x509-key-usage-extension->x509-key-usages

x509-private-key-usage-period?
make-x509-private-key-usage-period
x509-private-key-usage-period-not-before
x509-private-key-usage-period-not-after

make-x509-private-key-usage-period-extension
x509-private-key-usage-period-extension->x509-private-key-usage-period

x509-policy-qualifier-info?
x509-policy-qualifier-info-id
x509-policy-qualifier-info-qualifier
make-x509-policy-qualifier-info
x509-policy-qualifier-info->policy-qualifier-info

x509-notice-reference?
x509-notice-reference-organization
x509-notice-reference-notice-numbers
make-x509-notice-reference

x509-user-notice?
x509-user-notice-ref
x509-user-notice-explicit-text
make-x509-user-notice

x509-policy-information?
x509-policy-information-identifier
x509-policy-information-qualifiers
make-x509-policy-information
*policy-qualifier-type:cps*
*policy-qualifier-type:unotice*

make-x509-certificate-policies-extension
x509-certificate-policies-extension->x509-policy-informations

x509-basic-constraints?
make-x509-basic-constraints
x509-basic-constraints-ca?
x509-basic-constraints-path-length-constraint

make-x509-basic-constraints-extension
x509-basic-constraints-extension->x509-basic-constraints
-->
